@@ -24,41 +24,26 @@ fn find_subset_fast_only_positive(arr: Vec<u32>, value: usize, max_length: usize
     Ok(dp::find_subset_fast_only_positive(&arr, value, max_length))
 }
 
-/// Finds the integers from two vectors that sum to the same value.
-/// This method assumes that the two vectors have One-to-Many relationships.
-/// Each integer of the `key` vector corresponds to the multiple integers of the `targets` vector.
-/// # Arguments
-/// * `key` - An array.
-/// * `targets` - An array.
-/// * `max_length` - The maximum length of combinations of the answer.
-#[pyfunction]
-#[pyo3(text_signature = "(key, targets, max_target_length, /)")]
-fn sequence_matcher(mut key: Vec<i32>, mut targets: Vec<i32>, max_target_length: usize) -> PyResult<Vec<Vec<(i32, Vec<i32>)>>> {
-    use crate::dp_module::*;
-    Ok(dp::sequence_matcher(&mut key, &mut targets, max_target_length))
-}
-
 /// Finds the integers from two vectors that sum to the same value.
 /// This method assumes that the two vectors have Many-to-Many relationships.
 /// Each integer of the `keys` vector corresponds to the multiple integers of the `targets` vector.
-/// With this method, we can find some combinations of the integers.
-/// `n_shuffle` is the number of trials. For each case, `kays` are shuffled. 
+/// With this method, we can find combinations of the integers.
 /// # Arguments
 /// * `keys` - An array.
 /// * `targets` - An array.
-/// * `n_shuffle` - The number of trials.
+/// * `max_key_length` - An integer.
+/// * `max_target_length` - An integer.
 #[pyfunction]
-#[pyo3(text_signature = "(keys, targets, n_shuffle, /)")]
-fn sequence_matcher_m2m(mut keys: Vec<i32>, mut targets: Vec<i32>, n_shuffle: usize) -> PyResult<Vec<Vec<(Vec<i32>, Vec<i32>)>>> {
+#[pyo3(text_signature = "(keys, targets, max_key_length, max_target_length /)")]
+fn sequence_matcher(mut keys: Vec<i32>, mut targets: Vec<i32>, max_key_length: usize, max_target_length: usize) -> PyResult<Vec<Vec<(Vec<i32>, Vec<i32>)>>> {
     use crate::dp_module::*;
-    Ok(dp::sequence_matcher_m2m(&mut keys, &mut targets, n_shuffle))
+    Ok(dp::sequence_matcher(&mut keys, &mut targets, max_key_length, max_target_length))
 }
 
 #[pymodule]
 fn dpss(_py: Python, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(find_subset, m)?)?;
     m.add_function(wrap_pyfunction!(sequence_matcher, m)?)?;
-    m.add_function(wrap_pyfunction!(sequence_matcher_m2m, m)?)?;
     m.add_function(wrap_pyfunction!(find_subset_fast_only_positive, m)?)?;
 
     Ok(())
